;-------------------------------------------------------------------------------
; Init core lib functions
;
;   Copyright (C) 2008, 2009 Alesc
;   This program is free software: you can redistribute it and/or modify
;   it under the terms of the GNU General Public License.
;-------------------------------------------------------------------------------

;---------------------------------------
; InitCORElib
;
; Arguments: none
;
; Returns:
;   nothing
;
; Description:
;   Initializes core_lib. Must be called before
;   any other core function and module is called.
;
; Important globals initialized:
;   HOT_OK          
;   HOT_ERR         
;---------------------------------------
InitCORElib(prog_name)
{
    global HOT_OK, HOT_ERR

    HOT_OK := 0
    HOT_ERR := 1

    if prog_name =
    {
        Report("Error", "Can not initialize core lib. Invalid argumemts.")
        return %HOT_ERR%
    }

    ; get rid of last backslash if any
    tmp_path := RegExReplace(A_Temp, "\\$", "")
    Process, Exist
    tmp_path = %tmp_path%\%prog_name%%ErrorLevel%

    FileAppendEx(ErrorLevel, 1, prog_name . "_pid.dat")

    IfNotExist, %tmp_path%
        FileCreateDir, %tmp_path%

    If ErrorLevel
    {
        Report("Error", "Initalization of Core library failed.")
        return
    }

    return %tmp_path%
}
DeinitCORElib(prog_name)
{
    tmp_path := RegExReplace(A_Temp, "\\$", "")
    Process, Exist
    tmp_path = %tmp_path%\%prog_name%%ErrorLevel%

    FileRemoveDir, %tmp_path%, 1
    FileDelete, %prog_name%_pid.dat
}
;-------------------------------------------------------------------------------
; core functions
;-------------------------------------------------------------------------------

;---------------------------------------
; GetSelectedTxt
;
; Arguments: None
;
; Returns:
;   selected txt
;
; Description
;   Returns selected text with sending
;   copy to clipboard. Previous clipboard
;   is restored after use
;---------------------------------------
GetSelectedTxt()
{
    AutoTrim Off
    ClipboardOld = %ClipboardAll%
    Clipboard =
    Send, ^c
    ClipWait 1
    selected = %Clipboard%
    Clipboard = %ClipboardOld%

    return selected
}
;---------------------------------------
; RunCmd
;
; Arguments: cmd - command to execute
;            type - window type
;
; Returns:
;   Nothing
;
; Description:
;   Executes specific command and reports
;   if could not run
;---------------------------------------
RunCmd(cmd, type)
{
    global HOT_ERR, HOT_OK

    Run, %cmd%,,%type% UseErrorLevel
    if ErrorLevel = ERROR
    {
        Report("Error", "Error executing: " . cmd)
        return %HOT_ERR%
    }
    FileAppend
    return %HOT_OK%
}
;---------------------------------------
; RunCmdWait
;
; Arguments: cmd - command to execute
;            type - window type
;
; Returns:
;   Nothing
;
; Description:
;   Executes specific command and reports
;   if could not run. Waits cmd to finish.
;---------------------------------------
RunCmdWait(cmd, type)
{
    global HOT_ERR, HOT_OK

    RunWait, %cmd%,,%type% UseErrorLevel
    if ErrorLevel = ERROR
    {
        Report("Error", "Error executing: " . cmd)
        return %HOT_ERR%
    }
    return %HOT_OK%
}
;---------------------------------------
; IsInvalid
;
; Arguments: sel - txt selected
;
; Returns:
;   HOT_OK/HOT_ERR
;
; Description:
;   Checks if txt selected is valid.
;   Because RegExMatch/Autohotkey hangs 
;   on invalid chars. Dont know why and 
;   I dont really care
;---------------------------------------
IsValid(sel)
{
    global HOT_OK, HOT_ERR
    ; autohotkey hangs on these ones
    if InStr(sel, "(") or InStr(sel, ")") or InStr(sel, "*") or InStr(sel, ".")
    {
        Report("Warning", "Invalid string selected. '" . sel . "'")
        return %HOT_ERR%
    }
    return %HOT_OK%
}
;---------------------------------------
; Report
;
; Arguments: type - Error/Warning/Info
;            msg - text + variables to report
;
; Returns:
;   nothing
;
; Description:
;   Notifies user that some error/warning/info
;   happened in balloon message window. 
;   I dont use defines for type because
;   this means less code(global <define>
;   in functions)
;---------------------------------------
Report(type, msg)
{
    ; TODO get rid of global title
    global title
    if type = Error
    {
        TrayTip, %title%, %msg%, 5, 19
    }
    else if type = Warning
    {
        TrayTip, %title%, %msg%, 5, 18
    }
    else
    {
        TrayTip, %title%, %msg%, 2, 17
    }
}
;---------------------------------------
; WaitForFile
;
; Arguments: filename - filename with full path
;
; Returns:
;   HOT_OK / HOT_ERR
;
; Description:
;   Waits for a file to exist.
;---------------------------------------
WaitForFile(filename)
{
    global HOT_OK, HOT_ERR

    loop, 3600
    {
        IfExist, %filename%
            return %HOT_OK%

        Sleep, 1
    }
    Report("Error", "Timeout waiting for file[" . filename . "]")
    return %HOT_ERR%
}
;---------------------------------------
; WaitForFileEx
;
; Arguments: filename - filename with full path
;            delay - timeout between checks. milliseconds
;            loops - number of loops
;
; Returns:
;   HOT_OK / HOT_ERR
;
; Description:
;   Waits for a file to exist with
;   specified sleeps between checks.
;---------------------------------------
WaitForFileEx(filename, delay, loops, reports)
{
    global HOT_OK, HOT_ERR

    loop, %loops%
    {
        IfExist, %filename%
            return %HOT_OK%

        Sleep, %delay%
    }
    if reports = 1
        Report("Error", "Timeout waiting for file[" . filename . "]")
    return %HOT_ERR%
}
;---------------------------------------
; FindPath
;
; Arguments: selected - selected text
;
; Returns:
;   file/nothing
;
; Description:
;   Checks if selected file exists as it is
;   or exists in rootPath. Recurse only one
;   time...more could take forever.
;---------------------------------------
FindPath(selected, rootPath)
{
    ; cleanup any spaces before/after filename
    file := RegExReplace(selected, "^\s+([^\s]+)\s+$", "$1")

    nPaths = 0
    IfNotExist, %file%
    {
        file = %rootPath%%file%
        IfExist, %file%
            return file

        loop, %rootPath%*.*, 2, 0
        {
            paths%A_Index% = %A_LoopFileFullPath%
            nPaths++
            file = %A_LoopFileFullPath%\%selected%
            IfExist, %file%
                return file
        }
        loop, %nPaths%
        {
            rootPath := paths%A_Index%
            loop, %rootPath%\*.*, 2, 0
            {
                file = %A_LoopFileFullPath%\%selected%
                IfExist, %file%
                    return file
            }
        }
        Report("Error", selected . " not found.")
        return
    }

    return file
}
;---------------------------------------
; HintDisplay
;
; Arguments: hint
;
; Returns:
;   nothing
;
; Description:
;   
;---------------------------------------
HintDisplay(hint)
{
    Report("", hint)
}
;---------------------------------------
; CreateTmpFile
;
; Arguments: tmp_path - path of file (optional)
;            tmp_filename - prefix of file (optional)
;
; Returns:
;   filename
;
; Description:
;   Creates a temporary filename. Format:
;   <tmp_path>\<tmp_filename>TPR_<current_time>_<pid>_<rand999>.tmp
;---------------------------------------
CreateTmpFile(tmp_path, tmp_filename)
{
    global HOT_ERR, HOT_OK

    ; with this call pid is saved to ErrorLevel
    Process, Exist
    pid_piece = %ErrorLevel%
    time_piece = %A_Now%
    Random, random_piece, 1, 999

    ; get rid of last backslash if any
    tmp_path := RegExReplace(tmp_path, "\\$", "")
    if tmp_path
        tmp_file = %tmp_path%\
    else
        tmp_file = .\

    ; Time Pid Random
    tmp_file = %tmp_file%%tmp_filename%TPR_%time_piece%_%pid_piece%_%random_piece%.tmp

    return %tmp_file%
}
DeleteTmpFiles(tmp_path, tmp_filename)
{
    ; get rid of last backslash if any
    tmp_path := RegExReplace(tmp_path, "\\$", "")
    if tmp_path
        tmp_file = %tmp_path%\
    else
        tmp_file = .\

    FileDelete, %tmp_file%%tmp_filename%TPR_*.tmp
}
;---------------------------------------
; FileAppendEx
;
; Arguments: append_line - line to append
;            new_file_flag - 1/0...new file/append to old
;            target_file - 
;
; Returns:
;   HOT_OK / HOT_ERR
;
; Description:
;   Wrapper around FileAppend.
;---------------------------------------
FileAppendEx(append_line, new_file_flag, target_file)
{
    global HOT_OK, HOT_ERR
    DbgIn(A_ThisFunc)

    If new_file_flag = 1
    {
        DbgPrint("Delete previous file [" . target_file . "].")
        FileDelete, %target_file%
    }

    FileAppend, %append_line%, %target_file%
    If ErrorLevel
    {
        DbgPrint("Faild to append [" . append_line . "] to [" . target_file . "].")
        Report("Error", "Failed to append [" . append_line . "] to file [" . target_file . "].")
        DbgOut(A_ThisFunc)
        return %HOT_ERR%
    }
    DbgOut(A_ThisFunc)
    return %HOT_OK%
}
FileCreateDirEx(dir_name)
{
    global HOT_OK, HOT_ERR

    FileCreateDir, %dir_name%
    if ErrorLevel
    {
        Report("Error", "Failed to create [" . dir_name . "] direcotry.")
        return %HOT_ERR%
    }
    return %HOT_OK%
}
FileCopyEx(source, destination, overwrite)
{
    global HOT_OK, HOT_ERR

    FileCopy, %source%, %destination%, %overwrite%
    if ErrorLevel > 0
    {
        Report("Error", "Failed to copy [" . source . "] to [" . destination . "].")
        return %HOT_ERR%
    }
    return %HOT_OK%
}
RegWriteEx(value_type, root_key, sub_key, value_name, value)
{
    global HOT_OK, HOT_ERR

    RegWrite, %value_type%, %root_key%, %sub_key%, %value_name%, %value%
    if ErrorLevel
    {
        Report("Error", "Failed to write value [" . value_name . "] [" . value . "]to windows registry.")
        return %HOT_ERR%
    }

    return %HOT_OK%
}
RegReadEx(root_key, sub_key, value_name, ByRef value)
{
    global HOT_OK, HOT_ERR

    RegRead, value, %root_key%, %sub_key%, %value_name%
    if ErrorLevel
    {
        Report("Error", "Failed to read value [" . value_name . "] from windows registry.")
        return %HOT_ERR%
    }

    return %HOT_OK%
}
FileRemoveDirEx(directory, recurse)
{
    global HOT_OK, HOT_ERR

    FileRemoveDir, %directory%, %recurse%
    if ErrorLevel
    {
        Report("Error", "Failed to remove directory [" . directory . "].")
        return %HOT_ERR%
    }

    return %HOT_OK%
}
RegDeleteEx(root_key, sub_key)
{
    global HOT_OK, HOT_ERR

    RegDelete, %root_key%, %sub_key%
    if ErrorLevel
    {
        Report("Error", "Failed to remove [" . sub_key . "] from registry.")
        return %HOT_ERR%
    }

    return %HOT_OK%
}
GetCaseSens(sensitive)
{
    ; set to empty for case sensitive match of functions etc.
    if sensitive = 1
        return

    return i
}
GetFile(files, index)
{
    loop, parse, files, ";"
    {
        if A_Index = %index%
            return A_LoopField
    }
    Report("Error", "File not found on position " . index . ".")
    return
}
CntFiles(files)
{
    numberFiles = 0
    files := RegExReplace(files, ";$", "")
    loop, parse, files, ";"
        numberFiles := numberFiles +1

    return %numberFiles%
}
;HOT_ERR if no update
;HOT_OK if update available
GetUpdate(title, version, tmp_path)
{
    global HOT_OK, HOT_ERR

    position := RegExMatch(version, "(\d+)\.(\d+)", line)
    if (position = 0)
    {
        Report("Error", "Wrong input version.")
        return %HOT_ERR%
    }
    oldMajorVer = %line1%
    oldMinorVer = %line2%

    tmp_file := CreateTmpFile(tmp_path, "proj_web_")
    web_page = http://sourceforge.net/projects/%title%/

    UrlDownloadToFile, %web_page%, %tmp_file%
    if ErrorLevel
    {
        Report("Error", "Failed to retrieve web page [" . web_page . "].")
        return %HOT_ERR%
    }

    FileRead, web_content, %tmp_file%
    if ErrorLevel
    {
        Report("Error", "Failed to read web page from file [" . tmp_file . "].")
        return %HOT_ERR%
    }

    position := RegExMatch(web_content, "i)(" . title . " - " . title . "_((\d+)\.(\d+)))", line)
    if (position = 0)
    {
        Report("Error", "Failed to retrieve version from page. Improper web content.")
        return %HOT_ERR%
    }

    newMajorVer = %line3%
    newMinorVer = %line4%

    if ((oldMajorVer > %newMajorVer%) or ((oldMajorVer = newMajorVer) and (oldMinorVer >= newMinorVer)))
    {
        Report("", "No update available.")
        return %HOT_ERR%
    }

    Report("", "New version " . line2 . " available. Go to " . web_page . " to download.")
    return %HOT_OK%
}

LaunchProjWWW:
    Run http://%ProjWWW%.sourceforge.net/
return
